////////////////////////////////////////////////////////////////////////////////
//
// (C) 2010 Sarah Eris Horsley Caffee
//
// mandelbrot - A simple Mandelbrot Set viewer.
//
// Fractal.h
// Fractal calculator virtual class definition.  Actual fractals should 
// implement this.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef FRACTAL_H
#define FRACTAL_H

#include <map>
#include <string>

// Need to define this via the pre-processor because if it's a staticaly
// initialized const string we still have no guarantee that it will be 
// initialized before it is needed by a FractalCreator class.
// Do the same in all derived classes.

class Fractal
   {
   public:
   Fractal();

   virtual bool calc_set() = 0;
   virtual bool init(SDL_Surface * surf) = 0;

   double get_re_min() const;
   double set_re_min(const double r_min);

   double get_im_max() const;
   double set_im_max(const double i_max);

   double get_size() const;
   double set_size(const double s);

   unsigned int get_max_iter() const;
   unsigned int set_max_iter(const unsigned int iter);
   unsigned int inc_max_iter();
   unsigned int dec_max_iter();

   void set_calc_needed();

   virtual std::string const & get_display_name() const;
   virtual std::string const & get_fractal_name() const;


   virtual bool get_option(std::string option, void * value) const {};
   virtual bool set_option(std::string option, void * value) {};



   protected:

   virtual void draw_pixel(int x, int y, Uint32 * color);
   
   // fractal_name is the basic name of ths fractal, such as "Mandelbrot set"
   //   or "Julia set"
   // display_name is probably the same, but might have a bit of extra info,
   //   such as "Julia set K=(1,2)
   std::string display_name;
   std::string fractal_name;
   
   double re_min;
   double im_max;
   double size;
   unsigned int max_iter;
   bool calc_needed;
   
   SDL_Surface * surface;
   
   };

class FractalCreator
   {
   public:
   virtual Fractal * create() const = 0;
   };

std::map <std::string, FractalCreator *> & get_Fractal_map();

#endif
